#include "common.h"
#include "com.h"

#define TIMEOUT 5000

// Useful serial functions //
BOOL Write(HANDLE hFile, LPCVOID ptr, DWORD byte)
{
	DWORD bytes;
	DWORD tick;
	DWORD wrote = 0;

	tick = GetTickCount();

	while(wrote < byte)
	{
		WriteFile(hFile, (char*)ptr + wrote, byte - wrote, &bytes, NULL);
		wrote += bytes;
		if((tick + TIMEOUT) < GetTickCount())	// TIMEOUT!!
			return TRUE;
	}

	return FALSE;
}

BOOL WriteByte(HANDLE hFile, BYTE byte)
{
	return Write(hFile, &byte, 1);
}

BOOL Read(HANDLE hFile, LPVOID ptr, DWORD byte)
{
	DWORD bytes;
	DWORD read = 0;
	DWORD tick;

	tick = GetTickCount();

	while(read < byte)
	{
		ReadFile(hFile, (char*)ptr + read, byte - read, &bytes, NULL);
		read += bytes;
		if((tick + TIMEOUT) < GetTickCount())	// TIMEOUT!!
			return TRUE;
	}

	return FALSE;
}

WORD ReadByte(HANDLE hFile)
{
	BYTE ret;

	if(Read(hFile, &ret, 1))
		return 0x0100;

	return (WORD)ret;
}

// COM functions
BOOL SetBaudrate(HANDLE hcom, DWORD rate)
{
	DCB dcb;
	BOOL result;

	ZeroMemory(&dcb, sizeof(dcb));
	dcb.DCBlength = sizeof(dcb);
	GetCommState(hcom, &dcb);

	dcb.BaudRate = rate;

	PurgeComm(hcom, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
	result = SetCommState(hcom, &dcb);
	PurgeComm(hcom, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);

	return result;
}

HANDLE StartSerialTransfer(LPCTSTR comport)
{
	HANDLE hcom;

	if((hcom = CreateFile(comport, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL)) == INVALID_HANDLE_VALUE)
		return NULL;

	// Setup Buffer Size
	SetupComm(hcom, 4096, 4096);

	// Set TimeOut
	{
		COMMTIMEOUTS cto;

		cto.ReadIntervalTimeout = 10;
		cto.ReadTotalTimeoutMultiplier = 1;
		cto.ReadTotalTimeoutConstant = 100;
		cto.WriteTotalTimeoutMultiplier = 1;
		cto.WriteTotalTimeoutConstant = 100;
		SetCommTimeouts(hcom, &cto);
	}

	// Set BaudRate, StopBits, BitSize, Parity....
	{
		DCB dcb;

		ZeroMemory(&dcb, sizeof(dcb));
		dcb.DCBlength = sizeof(dcb);
		GetCommState(hcom, &dcb);

		dcb.StopBits = ONESTOPBIT;
		dcb.Parity = EVENPARITY;
		if(dcb.Parity != NOPARITY)
			dcb.fParity = TRUE;
		else
			dcb.fParity = FALSE;
		dcb.ByteSize = 8;

		dcb.EofChar = 0;
		dcb.ErrorChar = 0;
		// what is brk?
		dcb.fErrorChar = TRUE;	// needed?
		dcb.EvtChar = 0;
		dcb.XonChar = 0;
		dcb.XoffChar = 0;

		dcb.XoffLim = 0;
		dcb.XonLim = 0;
		SetCommState(hcom, &dcb);
	}

	SetBaudrate(hcom, 600);	// 600ċ@Ƃ̌݊?mȂB

	// need?
	PurgeComm(hcom, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);

	return hcom;
}
