#include "common.h"
#include "com.h"
#include "kcomm.h"
#include "mainWnd.h"
#include "filesWnd.h"

HWND hFilesWnd = NULL;

void getFileResCallback(DWORD pos, LPARAM lpdata)
{
	gFRCdata* data = (gFRCdata*)lpdata;

	{
		MSG msg;
		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
			DispatchMessage(&msg);
	}

	SendMessage(GetDlgItem(data->hwnd, IDC_FILEPROGRESS), PBM_SETPOS, pos * 1000 / data->size, 0);
	{
		char tmp[256];

		wsprintf(tmp, "%s %d%% (%u/%u)", data->name, pos * 100 / data->size, pos, data->size);
		SetDlgItemText(data->hwnd, IDC_FILESTATUS, tmp);
	}

	return;
}

BYTE downloadFile(HANDLE hcom, char* out, char* name, HWND hWnd)
{
	BYTE* cmd;
	DWORD size;

	{
		char tmp[256];

		wsprintf(tmp, "GetFileInfo %s...", name);
		SetDlgItemText(hWnd, IDC_FILESTATUS, tmp);
	}

	// GetFileSize
	{
		BYTE* res;
		DWORD ressize;

		cmd = halloc(2 + strlength(name) + 1);
		cmd[0] = 0x00;
		cmd[1] = 0x10;
		lstrcpy(cmd + 2, name);
		if(sendCommand(hcom, cmd, 2 + lstrlen(name) + 1, FALSE))
		{
			Log("getFile->SendFileName failed\r\n");
			hfree(cmd);
			return 0;
		}
		{
			BYTE ret = (BYTE)ReadByte(hcom);
			if(ret != 0x06)	// 0x18 = NoFile
			{
				Log("getFile: SFN NAK[");
				LogHex(ret);
				Log("]\r\n");
				hfree(cmd);
				return 0;
			}
		}
		cmd[0] = 0x02;
		cmd[1] = 0x12;
		if(sendCommand(hcom, cmd, 2, FALSE))
		{
			Log("getFile->GetInfo failed\r\n");
			hfree(cmd);
			return 0;	// for prevent leak, etc.
		}
		if(ReadByte(hcom) != 0x06)
		{
			Log("getFile: GI NAK\r\n");
			hfree(cmd);
			return 0;
		}

		hfree(cmd);

		if(getResult(hcom, &res, &ressize))
		{
			Log("getFileInfo: getResult failed\r\n");
			return 0;
		}

		{
			BYTE* b = res + strlength(res) + 1 + 4;

			size = b[0] << 24 | b[1] << 16 | b[2] << 8 | b[3];
		}

		hfree(res);
	}

	{
		char tmp[256];

		wsprintf(tmp, "Query %s...", name);
		SetDlgItemText(hWnd, IDC_FILESTATUS, tmp);
	}

	cmd = halloc(2 + strlength(name) + 1);
	cmd[0] = 0x02;
	cmd[1] = 0x10;
	strcopy(cmd + 2, name);
	if(sendCommand(hcom, cmd, 2 + strlength(name) + 1, FALSE))
		return 0;

	{
		WORD retval = ReadByte(hcom);

		if(retval == 0x06)	// ACK
		{
			BYTE* data;
			DWORD dsize;
			gFRCdata gfrcd = {hWnd, size, name};

			if(getResultWC(hcom, &data, &dsize, getFileResCallback, (LPARAM)&gfrcd))
			{
				Log("getResult failed\r\n");
				return 0;
			}

			SetDlgItemText(hWnd, IDC_FILESTATUS, "Saving");

			{
				HANDLE hf;
				DWORD written;

				if((hf = CreateFile(out, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL)) == INVALID_HANDLE_VALUE)
				{
					hfree(data);
					Log("CreateFile failed!?\r\n");
					return 0;
				}

				if(!WriteFile(hf, data, dsize, &written, NULL))
					Log("WriteFile failed!?\r\n");

				CloseHandle(hf);
			}

			SetDlgItemText(hWnd, IDC_FILESTATUS, "Done");

			hfree(data);
		}

		return (BYTE)retval;
	}
}

BYTE deleteFile(HANDLE hcom, char* name, HWND hWnd)
{
	BYTE* cmd;
	DWORD size;

	// GetFileSize(for exist check)
	if(0)
	{
		BYTE* res;
		DWORD ressize;

		cmd = halloc(2 + strlength(name) + 1);
		cmd[0] = 0x00;
		cmd[1] = 0x10;
		lstrcpy(cmd + 2, name);
		if(sendCommand(hcom, cmd, 2 + lstrlen(name) + 1, FALSE))
		{
			Log("getFile->SendFileName failed\r\n");
			hfree(cmd);
			return 0;
		}
		if(ReadByte(hcom) != 0x06)
		{
			Log("getFile: SFN NAK\r\n");
			hfree(cmd);
			return 0;
		}
		cmd[0] = 0x02;
		cmd[1] = 0x12;
		if(sendCommand(hcom, cmd, 2, FALSE))
		{
			Log("getFile->GetInfo failed\r\n");
			hfree(cmd);
			return 0;	// for prevent leak, etc.
		}
		if(ReadByte(hcom) != 0x06)
		{
			Log("getFile: GI NAK\r\n");
			hfree(cmd);
			return 0;
		}

		hfree(cmd);

		if(getResult(hcom, &res, &ressize))
		{
			Log("getFileInfo: getResult failed\r\n");
			return 0;
		}

		{
			BYTE* b = res + strlength(res) + 1 + 4;

			size = b[0] << 24 | b[1] << 16 | b[2] << 8 | b[3];
		}

		hfree(res);
	}

	Log("SaveFile(0byte)...");

	if(!(cmd = halloc(2 + strlength(name) + 1 + 4)))
		return 0;

	cmd[0] = 0x01;
	cmd[1] = 0x10;
	strcopy(cmd + 2, name);
	cmd[2 + strlength(name) + 1 + 0] = 0;
	cmd[2 + strlength(name) + 1 + 1] = 0;
	cmd[2 + strlength(name) + 1 + 2] = 0;
	cmd[2 + strlength(name) + 1 + 3] = 0;

	if(sendCommand(hcom, cmd, 2 + strlength(name) + 1 + 4, FALSE))
	{
		hfree(cmd);
		return 0;
	}
	{
		BYTE ret = (BYTE)ReadByte(hcom);

		if(ret != 0x06)	// 0x18 = NoFile
		{
			Log("NAK[");
			LogHex(ret);
			Log("]");
			hfree(cmd);
			return 0;
		}
	}
 
	hfree(cmd);

	Log("data...");

	if(sendCommand(hcom, NULL, 0, TRUE))
		return 0;
	return 0x06;
}

void getFileListInfo(HWND hfw)
{
	HANDLE hcom;

	EnableWindow(GetDlgItem(hfw, IDC_GETFILELIST), FALSE);
	EnableWindow(GetDlgItem(hfw, IDC_GETFILEINFO), FALSE);
	EnableWindow(GetDlgItem(hfw, IDOK), FALSE);

	if(beginComm(&hcom))
	{
		Log("Cannot begin comm\r\n");
		return;
	}

	Log("Receiving info...");
	{
		WORD i;
		WORD files = ListView_GetItemCount(GetDlgItem(hfw, IDC_FILELIST));
		char name[256];

		SetDlgItemText(hfw, IDC_STATUS, "Getting Fileinfo...");
		SendMessage(GetDlgItem(hfw, IDC_PROGRESS), PBM_SETRANGE, 0, MAKELPARAM(0, files));
		SendMessage(GetDlgItem(hfw, IDC_PROGRESS), PBM_SETPOS, 0, 0);

		for(i = 0; i < files; i++)
		{
			LV_ITEM lvi = {0};
			BYTE* res;
			BYTE* cmd;
			DWORD size;
			int ii;

			if(GetAsyncKeyState(VK_ESCAPE) & 0x8000)
				break;

			{
				MSG msg;
				if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
					DispatchMessage(&msg);
			}

			// for GettingSort bug
			{
				LV_FINDINFO lvf;

				lvf.flags = LVFI_PARAM;
				lvf.lParam = i;
				ii = ListView_FindItem(GetDlgItem(hfw, IDC_FILELIST), -1, &lvf);

				ListView_GetItemText(GetDlgItem(hfw, IDC_FILELIST), ii, 0, name, sizeof(name));
			}

			cmd = halloc(2 + strlength(name) + 1);
			cmd[0] = 0x00;
			cmd[1] = 0x10;
			strcopy(cmd + 2, name);
			if(sendCommand(hcom, cmd, 2 + strlength(name) + 1, FALSE))
			{
				Log("getFileInfo->SendFileName failed\r\n");
				break;	// for prevent leak, etc.
			}
			if(ReadByte(hcom) != 0x06)
			{
				Log("getFileInfo: SFN NAK\r\n");
				break;
			}
			cmd[0] = 0x02;
			cmd[1] = 0x12;
			if(sendCommand(hcom, cmd, 2, FALSE))
			{
				Log("getFileInfo->GetInfo failed\r\n");
				break;	// for prevent leak, etc.
			}
			if(ReadByte(hcom) != 0x06)
			{
				Log("getFileInfo: GI NAK\r\n");
				break;
			}

			hfree(cmd);

			if(getResult(hcom, &res, &size))
			{
				Log("getFileInfo: getResult failed\r\n");
				break;
			}

			{
				BYTE* b;

				b = res + strlength(res) + 1 + 4;

				{
					char tmp[16];

					wsprintf(tmp, "%d", b[0] << 24 | b[1] << 16 | b[2] << 8 | b[3]);
					ListView_SetItemText(GetDlgItem(hfw, IDC_FILELIST), ii, 2, tmp);
				}
				b += 4;

				{
					char datestr[32];
					struct{
						struct{
							int y;
							int m;
							int d;
						}date;
						struct{
							int h;
							int m;
							int s;
						}time;
					}stamp;

					stamp.date.y = atodn(b + 0, 4);
					stamp.date.m = atodn(b + 4, 2);
					stamp.date.d = atodn(b + 6, 2);
					stamp.time.h = atodn(b + 8, 2);
					stamp.time.m = atodn(b + 10, 2);
					stamp.time.s = atodn(b + 12, 2);
					wsprintf(datestr, "%04d/%02d/%02d %02d:%02d:%02d", stamp.date.y, stamp.date.m, stamp.date.d, stamp.time.h, stamp.time.m, stamp.time.s);
					ListView_SetItemText(GetDlgItem(hfw, IDC_FILELIST), ii, 3, datestr);
				}
				b += strlength(b) + 1;

				{
					char tmp[64];
					int j;

					tmp[0] = '\0';
					for(j = 0; j < 16; j++)
					{
						char hex[4];
						wsprintf(hex, "%02X ", b[j]);
						strplus(tmp, hex);
					}

					ListView_SetItemText(GetDlgItem(hfw, IDC_FILELIST), ii, 4, tmp);
				}
			}

			hfree(res);

			{
				char tmp[80];

				wsprintf(tmp, "Getting Fileinfo... %d/%d", i, files);
				SetDlgItemText(hfw, IDC_STATUS, tmp);
			}
			SendMessage(GetDlgItem(hfw, IDC_PROGRESS), PBM_SETPOS, i, 0);
		}
	}

	EnableWindow(GetDlgItem(hfw, IDC_GETFILELIST), TRUE);
	EnableWindow(GetDlgItem(hfw, IDC_GETFILEINFO), TRUE);
	EnableWindow(GetDlgItem(hfw, IDOK), TRUE);

	if(GetAsyncKeyState(VK_ESCAPE) & 0x8000)
		SetDlgItemText(hfw, IDC_STATUS, "GetFileInfo aborted by user.");
	else
		SetDlgItemText(hfw, IDC_STATUS, "GetFileInfo accomplished.");

	if(endComm(hcom))
	{
		Log("Illegal endComm\r\n");
	}

	return;
}

void getFileList(HWND hfw)
{
	HANDLE hcom = NULL;

	EnableWindow(GetDlgItem(hfw, IDOK), FALSE);
	EnableWindow(GetDlgItem(hfw, IDC_GETFILELIST), FALSE);

	Log("<<GetFiles>>\r\n");

	Log("beginComm...");
	if(beginComm(&hcom))
	{
		Log("failed\r\n");
		goto error;
	}
	Log("OK\r\n");

	{
		BYTE* list;
		DWORD size;

		SetDlgItemText(hfw, IDC_STATUS, "Getting Filelist...");
		SendMessage(GetDlgItem(hfw, IDC_PROGRESS), PBM_SETRANGE, 0, MAKELPARAM(0, 1));
		SendMessage(GetDlgItem(hfw, IDC_PROGRESS), PBM_SETPOS, 0, 0);
		SendMessage(GetDlgItem(hfw, IDC_FILELIST), LVM_DELETEALLITEMS, 0, 0);

		Log("GetList...");
		{
			BYTE buf[2] = {0x02, 0x14};

			if(sendCommand(hcom, buf, 2, FALSE))
			{
				Log("failed\r\n");
				goto error;
			}
		}
		if(ReadByte(hcom) != 0x06)
		{
			Log("NAK\r\n");
			goto error;
		}

		Log("Receiving...");
		if(getResult(hcom, &list, &size))
		{
			Log("failed\r\n");
			goto error;
		}
		Log("OK\r\n");

		{
			WORD i;
			WORD files = (list[0] << 8) | list[1];
			BYTE* b = list + 2;

			for(i = 0; i < files; i++)
			{
				LV_ITEM lvi = {0};

				lvi.mask = LVIF_TEXT | LVIF_PARAM;
				lvi.pszText = b;
				lvi.iItem = i;
				lvi.lParam = i;
				ListView_InsertItem(GetDlgItem(hfw, IDC_FILELIST), &lvi);

				{
					char ext[16];
					DWORD l = strlength(b);
					BYTE* x = b + l - 1;

					while(l && *x != '.')
						l--, x--;

					strcopy(ext, x + 1);
					ListView_SetItemText(GetDlgItem(hfw, IDC_FILELIST), i, 1, ext);
				}

				b += strlength(b) + 1;
			}
		}

		EnableWindow(GetDlgItem(hfw, IDOK), TRUE);
		EnableWindow(GetDlgItem(hfw, IDC_GETFILELIST), TRUE);
		EnableWindow(GetDlgItem(hfw, IDC_GETFILEINFO), TRUE);
	}
	Log("OK\r\n");

	SetDlgItemText(hfw, IDC_STATUS, "Closing connection...");

	Log("Close connection ");
	if(EndSerialTransfer(hcom))
	{
		Log("illegally!\r\n");
		goto error;
	}

	Log("nicely.\r\n");

	SetDlgItemText(hfw, IDC_STATUS, "GetFileList accomplished.");

	return;
error:
	if(hcom)
		if(EndSerialTransfer(hcom))
			CloseHandle(hcom);

	EnableWindow(GetDlgItem(hfw, IDOK), TRUE);
	EnableWindow(GetDlgItem(hfw, IDC_GETFILELIST), TRUE);
	EnableWindow(GetDlgItem(hfw, IDC_GETFILEINFO), TRUE);

	SetDlgItemText(hfw, IDC_STATUS, "GetFileList aborted by Error.");
	return;
}

int CALLBACK sortFileListCallBack(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	LV_FINDINFO lvf;
	int ii1, ii2;
	static char buf1[256], buf2[256];
	int ret;

	lvf.flags = LVFI_PARAM;
	lvf.lParam = lParam1;
	ii1 = ListView_FindItem(GetDlgItem(hFilesWnd, IDC_FILELIST), -1, &lvf);

	lvf.lParam = lParam2;
	ii2 = ListView_FindItem(GetDlgItem(hFilesWnd, IDC_FILELIST), -1, &lvf);

	ListView_GetItemText(GetDlgItem(hFilesWnd, IDC_FILELIST), ii1, LOWORD(lParamSort), buf1, sizeof(buf1));
	ListView_GetItemText(GetDlgItem(hFilesWnd, IDC_FILELIST), ii2, LOWORD(lParamSort), buf2, sizeof(buf2));

	if(HIBYTE(HIWORD(lParamSort)))
	{
		// NumericSort
		DWORD s1, s2;
		int l1, l2;

		l1 = lstrlen(buf1);
		l2 = lstrlen(buf2);
		if(!l1 && !l2)
			ret = 0;
		else
			if(!l1 && l2)
				ret = -1;
			else
				if(l1 && !l2)
					ret = 1;
				else
				{
					s1 = atod(buf1);
					s2 = atod(buf2);

					if(s1 == s2)
						ret = 0;
					if(s1 < s2)
						ret = -1;
					else
						ret = 1;
				}
	}else
	{
		// ASCII Sort
		ret = lstrcmp(buf1, buf2);
	}
	if(LOBYTE(HIWORD(lParamSort)))
		ret *= -1;

	return ret;
}

BOOL CALLBACK FilesWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	static char outdir[256];

	switch(msg)
	{
	case WM_INITDIALOG:
		SendMessage(GetDlgItem(hWnd, IDC_FILELIST), LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT, 0xFFFFFFFF);

		{
			LV_COLUMN lvc = {0};

			lvc.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_FMT;
			lvc.cx = 200;
			lvc.pszText = "Filename";
			lvc.fmt = LVCFMT_LEFT;
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_FILELIST), 0, &lvc);
			lvc.cx = 60;
			lvc.pszText = "Ext";
			lvc.fmt = LVCFMT_LEFT;
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_FILELIST), 1, &lvc);
			lvc.cx = 60;
			lvc.pszText = "Size";
			lvc.fmt = LVCFMT_RIGHT;
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_FILELIST), 2, &lvc);
			lvc.cx = 120;
			lvc.pszText = "Time";
			lvc.fmt = LVCFMT_LEFT;
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_FILELIST), 3, &lvc);
			lvc.cx = 100;
			lvc.pszText = "ExtraData";
			lvc.fmt = LVCFMT_LEFT;
			ListView_InsertColumn(GetDlgItem(hWnd, IDC_FILELIST), 4, &lvc);
		}

		SendMessage(GetDlgItem(hWnd, IDC_FILEPROGRESS), PBM_SETRANGE, 0, MAKELPARAM(0, 1000));
		SetDlgItemText(hWnd, IDC_FILESTATUS, "");
		SetDlgItemText(hWnd, IDC_STATUS, "Click [GetFileList].");

		outdir[0] = '\0';

		return TRUE;	// set focus
	case WM_LBUTTONDOWN:
		SendMessage(hWnd, WM_NCLBUTTONDOWN, HTCAPTION, 0);
		break;
	case WM_CLOSE:
		EndDialog(hWnd, 0);
		hFilesWnd = NULL;
		break;
	case WM_COMMAND:
		switch(HIWORD(wParam))
		{
		case 0:
			switch(LOWORD(wParam))
			{
			case IDOK:
				PostMessage(hWnd, WM_CLOSE, 0, 0);
				break;
			case IDC_GETFILELIST:
				getFileList(hWnd);
				break;
			case IDC_GETFILEINFO:
				getFileListInfo(hWnd);
				break;
			case IDC_OUTDIR:
				{
					BROWSEINFO bi = {0};
					LPITEMIDLIST il;
					char path[MAX_PATH];

					bi.hwndOwner = hWnd;
					bi.pidlRoot = NULL;
					bi.lpszTitle = "Download to...";
					bi.ulFlags = BIF_RETURNONLYFSDIRS;

					if(il = SHBrowseForFolder(&bi))
					{
						SHGetPathFromIDList(il, path);
						CoTaskMemFree(il);

						addEndYen(path);

						strcopy(outdir, path);
						SetDlgItemText(hWnd, IDC_OUTDIR, outdir);
					}
				}
				break;
			case ID_GETFILE:
				if(!strlength(outdir))
					SendMessage(hWnd, WM_COMMAND, MAKEWPARAM(IDC_OUTDIR, 0), 0);
				if(strlength(outdir))
				{
					HANDLE hcom;
					char port[16];
					DWORD baud;

					// start!
					GetDlgItemText(hMainWnd, IDC_PORT, port, 16);
					baud = GetDlgItemInt(hMainWnd, IDC_BAUDRATE, NULL, FALSE);

					Log("<<GetFile>>\r\n");
					Log("Open COM...");
					if(!(hcom = StartSerialTransfer(port)))
					{
						Log("failed\r\n");
						break;
					}
					Log("OK\r\n");

					Log("ChangeMode...");
					if(ChangeMode(hcom))
						goto getFileError;
					Log("OK\r\n");

					Log("ChangeBaud...");
					if(ChangeBaudrate(hcom, baud))
						goto getFileError;
					Log("OK\r\n");

					{
						int i;
						int count = 0;
						int selected = ListView_GetSelectedCount(GetDlgItem(hWnd, IDC_FILELIST));

						SendMessage(GetDlgItem(hWnd, IDC_PROGRESS), PBM_SETRANGE, 0, MAKELPARAM(0, selected));
						SendMessage(GetDlgItem(hWnd, IDC_PROGRESS), PBM_SETPOS, 0, 0);

						for(i = 0; i < ListView_GetItemCount(GetDlgItem(hWnd, IDC_FILELIST)); i++)
						{
							if(ListView_GetItemState(GetDlgItem(hWnd, IDC_FILELIST), i, LVIS_SELECTED) & LVIS_SELECTED)
							{
								char name[256];
								char fn[256];

								if(GetAsyncKeyState(VK_ESCAPE) & 0x8000)
									break;

								ListView_GetItemText(GetDlgItem(hWnd, IDC_FILELIST), i, 0, name, 256);

								strcopy(fn, outdir);
								strplus(fn, name);

								Log("Receiving ");
								Log(name);
								Log("...");

								{
									char tmp[256];

									wsprintf(tmp, "Getting(%d/%d) %s...", count, selected, name);
									SetDlgItemText(hWnd, IDC_STATUS, tmp);
									SendMessage(GetDlgItem(hWnd, IDC_PROGRESS), PBM_SETPOS, count, 0);
									count++;
								}

								{
									BYTE ret = downloadFile(hcom, fn, name, hWnd);
									switch(ret)
									{
									case 0x06:
										Log("OK\r\n");
										break;
									case 0x18:
										Log("NAK\r\n");
										break;
									default:
										Log("Unknown[");
										LogHex(ret);
										Log("]\r\n");
										break;
									}
								}

								{
									char tmp[256];

									wsprintf(tmp, "GetFin %s...", name);
									SetDlgItemText(hWnd, IDC_STATUS, tmp);
									SendMessage(GetDlgItem(hWnd, IDC_PROGRESS), PBM_SETPOS, count, 0);
								}
							}
						}
					}

					// kokorahen naosite.
getFileErrorResume:
					Log("Close connection ");
					if(EndSerialTransfer(hcom))
						Log("illegally!\r\n");
					else
						Log("nicely\r\n");

					if(GetAsyncKeyState(VK_ESCAPE) & 0x8000)
						SetDlgItemText(hWnd, IDC_STATUS, "GetFile aborted by user.");
					else
						SetDlgItemText(hWnd, IDC_STATUS, "GetFile accomplished.");
					SetDlgItemText(hWnd, IDC_FILESTATUS, "");
					SendMessage(GetDlgItem(hWnd, IDC_FILEPROGRESS), PBM_SETPOS, 0, 0);

					break;
getFileError:
					Log("failed!\r\n");
					goto getFileErrorResume;
				}
				break;
			case ID_DELETEFILE:
				{
					DWORD selected = ListView_GetSelectedCount(GetDlgItem(hWnd, IDC_FILELIST));
					char tmp[256];

					wsprintf(tmp, "YOU WILL LOST SELECTED %d FILE(S)!\n\nA R E  Y O U  S U R E ?", selected);

					if(MessageBox(hWnd, tmp, "LAST WARNING", MB_ICONWARNING | MB_YESNO) == IDYES)
					{
						int i;
						int count = 0;
						int selected = ListView_GetSelectedCount(GetDlgItem(hWnd, IDC_FILELIST));
						HANDLE hcom;

						Log("<<DELETE FILE>>\r\n");

						Log("beginComm...");
						if(beginComm(&hcom))
						{
							Log("failed\r\n");
							break;
						}
						Log("OK\r\n");

						SendMessage(GetDlgItem(hWnd, IDC_PROGRESS), PBM_SETRANGE, 0, MAKELPARAM(0, selected));
						SendMessage(GetDlgItem(hWnd, IDC_PROGRESS), PBM_SETPOS, 0, 0);

						for(i = 0; i < ListView_GetItemCount(GetDlgItem(hWnd, IDC_FILELIST)); i++)
						{
							if(ListView_GetItemState(GetDlgItem(hWnd, IDC_FILELIST), i, LVIS_SELECTED) & LVIS_SELECTED)
							{
								char name[256];

								if(GetAsyncKeyState(VK_ESCAPE) & 0x8000)
									break;

								{
									MSG msg;
									if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
										DispatchMessage(&msg);
								}

								ListView_GetItemText(GetDlgItem(hWnd, IDC_FILELIST), i, 0, name, 256);

								Log("Deleting ");
								Log(name);
								Log("...");

								{
									char tmp[256];

									wsprintf(tmp, "Deleting(%d/%d)...", count, selected);
									SetDlgItemText(hWnd, IDC_STATUS, tmp);
									SetDlgItemText(hWnd, IDC_FILESTATUS, name);
									SendMessage(GetDlgItem(hWnd, IDC_PROGRESS), PBM_SETPOS, count, 0);
									count++;
								}

								{
									BYTE ret = deleteFile(hcom, name, hWnd);
									switch(ret)
									{
									case 0x06:
										Log("OK\r\n");
										break;
									case 0x18:
										Log("NAK\r\n");
										break;
									default:
										Log("Unknown[");
										LogHex(ret);
										Log("]\r\n");
										break;
									}
								}

								{
									char tmp[256];

									wsprintf(tmp, "Deleted(%d/%d)", count, selected);
									SetDlgItemText(hWnd, IDC_STATUS, tmp);
									SendMessage(GetDlgItem(hWnd, IDC_PROGRESS), PBM_SETPOS, count, 0);
								}
							}
						}

						// kokorahen naosite.
						Log("Close connection ");
						if(EndSerialTransfer(hcom))
							Log("illegally!\r\n");
						else
							Log("nicely\r\n");

						if(GetAsyncKeyState(VK_ESCAPE) & 0x8000)
							SetDlgItemText(hWnd, IDC_STATUS, "DeleteFile aborted by user.");
						else
							SetDlgItemText(hWnd, IDC_STATUS, "DeleteFile accomplished.");
						SetDlgItemText(hWnd, IDC_FILESTATUS, "");

						// If don't wait, KEITAI will disconnect.
						Sleep(250);

						PostMessage(hWnd, WM_COMMAND, MAKEWPARAM(IDC_GETFILELIST, 0), 0);

						break;
					}
				}
				break;
			}
		}
		break;
	case WM_NOTIFY:
		switch((int)wParam)
		{
		case IDC_FILELIST:
			{
				LV_DISPINFO* lvi = (LV_DISPINFO*)lParam;

				if(lvi->hdr.code == LVN_COLUMNCLICK)
				{
					NM_LISTVIEW* nmlv = (NM_LISTVIEW*)lParam;
					static int sortcolumn = -1;
					static BOOL sortreverse = FALSE;

					if(nmlv->iSubItem != sortcolumn)
					{
						sortreverse = FALSE;
						sortcolumn = nmlv->iSubItem;
					}else
						sortreverse = sortreverse ? FALSE : TRUE;	// reverse flag

					{
						LV_COLUMN lvc;

						lvc.mask = LVCF_FMT;
						if(!ListView_GetColumn(GetDlgItem(hWnd, IDC_FILELIST), nmlv->iSubItem, &lvc))
							MessageBeep(MB_ICONERROR);
						if((lvc.fmt & 0x000F) == LVCFMT_RIGHT)	// What 0x4000?
							// NumericSort
							ListView_SortItems(GetDlgItem(hWnd, IDC_FILELIST), sortFileListCallBack, MAKELPARAM(nmlv->iSubItem, MAKEWORD(sortreverse, TRUE)));
						else
							// ASCII Sort
							ListView_SortItems(GetDlgItem(hWnd, IDC_FILELIST), sortFileListCallBack, MAKELPARAM(nmlv->iSubItem, MAKEWORD(sortreverse, FALSE)));
					}
				}
				break;
			}
		}
		break;
	case WM_CONTEXTMENU:
		if((HWND)wParam == GetDlgItem(hWnd, IDC_FILELIST) && ListView_GetSelectedCount(GetDlgItem(hWnd, IDC_FILELIST)))
		{
			HMENU hpm = LoadMenu(GetModuleHandle(NULL), MAKEINTRESOURCE(IDR_FILEPOPUP));
			HMENU hm = GetSubMenu(hpm, 0);

			TrackPopupMenu(hm, 0, LOWORD(lParam), HIWORD(lParam), 0, hWnd, NULL);
			DestroyMenu(hm);
			DestroyMenu(hpm);
		}
		break;
	}

	return FALSE;
}
