#include "common.h"
#include "com.h"
#include "mainWnd.h"
#include "kcomm.h"

DWORD kcomm_framesize;

BOOL sendCommandWC(HANDLE hCom, BYTE* ptr, DWORD byte, BOOL isdata, void (*callback)(DWORD, LPARAM), LPARAM lpdata)
{
	unsigned char tmp[5];
	DWORD i;
	DWORD left = byte;

	do
	{
		unsigned short sum = 0xFFFF;
		DWORD pktsize = (left > (DWORD)(kcomm_framesize - 5 - 3)) ? (kcomm_framesize - 5 - 3) : (WORD)left;

		if(callback)
			callback(byte - left, lpdata);

		tmp[0] = 0x01;
		if(isdata)
			tmp[1] = 0x80;	// data
		else
			tmp[1] = 0x00;	// command
		tmp[2] = (unsigned char)((1 + pktsize + 3) >> 8) & 0xFF;
		tmp[3] = (unsigned char)(1 + pktsize + 3) & 0xFF;
		tmp[4] = 0x02;
		if(Write(hCom, tmp, 5))
			return TRUE;	// head send failed

		for(i = 0; i < 5; i++)
			sum -= tmp[i];

		if(ptr && pktsize)	// for 0byte packet.
			if(Write(hCom, ptr, pktsize))
				return TRUE;	// data send failed

		for(i = 0; i < pktsize; i++)
			sum -= ((unsigned char*)ptr)[i];

		left -= pktsize;
		ptr += pktsize;

		if(!left)	// left=0 == last
			tmp[0] = 0x03;
		else
			tmp[0] = 0x17;
		sum -= tmp[0];
		tmp[1] = (unsigned char)(sum >> 8) & 0xFF;
		tmp[2] = (unsigned char)sum & 0xFF;
		if(Write(hCom, tmp, 3))
			return TRUE;	// foot send failed

		if(isdata)
			if(ReadByte(hCom) != 0x06)	// PacketACK?
				return TRUE;
	}
	while(left);

	return FALSE;
}

BOOL sendCommand(HANDLE hCom, BYTE* ptr, DWORD byte, BOOL isdata)
{
	return sendCommandWC(hCom, ptr, byte, isdata, NULL, 0);
}

BOOL getResultWC(HANDLE hCom, BYTE** retbuf, DWORD* retsize, void (*callback)(DWORD, LPARAM), LPARAM lpdata)
{
	WORD size;
	BYTE buf[2];
	BYTE* data = NULL;

	*retsize = 0;

	for(;;)
	{
		WORD chksum = 0xFFFF;
		WORD datacs;
		WORD i;

		if(callback)
			callback(*retsize, lpdata);

		// header check
		if(ReadByte(hCom) != 0x01)
			goto error;
		chksum -= 0x01;
		if(ReadByte(hCom) != 0x80)
			goto error;
		chksum -= 0x80;

		// read data size
		if(Read(hCom, buf, 2))
			goto error;

		size = (buf[0] << 8) | buf[1];
		data = hrealloc(data, *retsize + size);
		chksum -= buf[0];
		chksum -= buf[1];

		// read data
		if(ReadByte(hCom) != 0x02)
			goto error;
		size--;
		chksum -= 0x02;

		if(Read(hCom, data + *retsize, size))
			goto error;

		for(i = 0; i < size - 2; i++)
			chksum -= *(data + *retsize + i);
		datacs = (data[*retsize + size - 2] << 8) | data[*retsize + size - 1];

		if(chksum != datacs)
		{
			WriteByte(hCom, 0x08);	// obsolete: if bad, what command byte we should send?
			continue;
		}

		// 0x07 need? (MySync)
		WriteByte(hCom, 0x06);	// ACK

		*retsize += size - 3;

		if(data[*retsize] == 0x17)
			continue;
		if(data[*retsize] == 0x03)
			break;

		{
			char tmp[80];

			wsprintf(tmp, "Exception: in getResult: Illegal return char: [%02X]\r\nAbort function!", data[*retsize]);
			MessageBox(NULL, tmp, "Bug or error", MB_ICONERROR | MB_OK);
		}
		goto error;
	}

	*retbuf = data;

	return FALSE;
error:
	if(data)
		hfree(data);
	*retsize = 0;

	return TRUE;
}

BOOL getResult(HANDLE hCom, BYTE** retbuf, DWORD* retsize)
{
	return getResultWC(hCom, retbuf, retsize, NULL, 0);
}

BOOL ChangeMode(HANDLE hcom)
{
	BYTE buf[4];

retry:
	// Mode change?
	buf[0] = 0xD0;
	buf[1] = 0x01;
	buf[2] = 0x05;
	if(Write(hcom, buf, 3))
	{
		Log("D00105 Write Failed\r\n");
		return TRUE;
	}
	buf[2] = 0x06;
	{
		DWORD c = 0;
		WORD rb;

		while(c < 3)
		{
			if((rb = ReadByte(hcom)) == 0x0100)
			{
				Log("D00105 phase Read Failed\r\n");
				return TRUE;
			}
			// if(rb & 0xF0 == 0xC0)
			if(rb == 0x8E || (0xC0 <= rb && rb <= 0xCF))	// it is dialLockOff/antenna level
				continue;
			if(rb == 0x08)
			{
				Log("Retrying...");
				goto retry;
			}
			if(buf[c] != rb)
			{
				Log("D00105 phase Mismatch(");
				LogHex(rb);
				Log(")\r\n");
				return TRUE;
			}
			c++;
		}
	}
	// here, will not come 0xC* data(antenna level) NO LONGER??

	buf[0] = 0x05;
	if(Write(hcom, buf, 1))
	{
		Log("05 Write Failed\r\n");
		return TRUE;
	}
	if(Read(hcom, buf, 1))
	{
		Log("05 Read Failed\r\n");
		return TRUE;
	}
	if(buf[0] != 0x06)
	{
		Log("05 Data Error RET=");
		LogHex(buf[0]);
		Log("\r\n");
		return TRUE;
	}

	return FALSE;
}

int setFrameSize(HANDLE hcom, DWORD framesize)
{
	BYTE buf[6];

	buf[0] = 0x00;
	buf[1] = 0x03;
	buf[2] = 0x00;
	buf[3] = 0x00;
	buf[4] = (BYTE)((framesize >> 8) & 0xFF);
	buf[5] = (BYTE)(framesize & 0xFF);

	if(sendCommand(hcom, buf, 6, FALSE))
		return 1;
	if(ReadByte(hcom) != 0x06)
		return 2;

	kcomm_framesize = framesize;

	return 0;
}

// sub routine //
int ChangeBaudrate(HANDLE hcom, DWORD rate)
{
	unsigned char buf[32];

	// ChangeSpeed...?

	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0x00;
	buf[3] = 0x01;
	buf[4] = 0x00;
	buf[5] = 0x00;

	if(sendCommand(hcom, buf, 6, FALSE))
	{
		Log("CB1 Write failed\r\n");
		return 1;
	}
	if(ReadByte(hcom) != 0x06)
	{
		Log("CB1 NAK\r\n");
		return 2;
	}

	buf[0] = 0x00;
	buf[1] = 0x01;
	buf[2] = (unsigned char)(rate >> 24) & 0xFF;
	buf[3] = (unsigned char)(rate >> 16) & 0xFF;
	buf[4] = (unsigned char)(rate >> 8) & 0xFF;
	buf[5] = (unsigned char)rate & 0xFF;

	if(sendCommand(hcom, buf, 6, FALSE))
	{
		Log("CB2 Write failed\r\n");
		return 3;
	}
	if(ReadByte(hcom) != 0x06)
	{
		Log("CB2 NAK\r\n");
		return 4;
	}

	if(!SetBaudrate(hcom, rate))
	{
		Log("Warning: SetBaudrate failed\r\n");
	}

	if(setFrameSize(hcom, 4096))
	{
		Log("Framesize failed\r\n");
		return 5;
	}

	return 0;
}

BOOL EndSerialTransfer(HANDLE hcom)
{
	Log("EOT ");
	if(WriteByte(hcom, 0x04))
	{
		Log("Write failed\r\n");
		CloseHandle(hcom);
		return TRUE;
	}
	Log("sent.");

	// Ô
	PurgeComm(hcom, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);

	CloseHandle(hcom);

	return FALSE;
}

int beginComm(HANDLE* ret)
{
	HANDLE hcom;

	char port[16];
	DWORD baud;
	DWORD frame;

	// Get about COM/Baud info
	GetDlgItemText(hMainWnd, IDC_PORT, port, 16);
	baud = GetDlgItemInt(hMainWnd, IDC_BAUDRATE, NULL, FALSE);
	frame = GetDlgItemInt(hMainWnd, IDC_FRAMESIZE, NULL, FALSE);

	if(!(hcom = StartSerialTransfer(port)))
		return 1;

	if(ChangeMode(hcom))
	{
		CloseHandle(hcom);
		return 2;
	}

	if(ChangeBaudrate(hcom, baud))
	{
		WriteByte(hcom, 0x04);
		CloseHandle(hcom);
		return 3;
	}

	if(setFrameSize(hcom, frame))
	{
		WriteByte(hcom, 0x04);
		CloseHandle(hcom);
		return 4;
	}

	*ret = hcom;

	return 0;
}

BOOL endComm(HANDLE hcom)
{
	return EndSerialTransfer(hcom);
}
